package org.noote.robot_remote;

import org.noote.libs.bluetooth.SPPClient;
import org.noote.libs.roby.comm.BluetoothCommandProtocol;
import org.noote.libs.virtualpad.Virtual5DPad;
import org.noote.libs.virtualpad.VirtualTouchPad;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.PowerManager;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.FrameLayout;
import android.widget.Toast;

public class Remote_Activity extends Activity {

	static final String TAG = "Remote_Activity";
	static final String BT_ADDRESS_DEVICE = "BtAddressDevice";

	private String _sAddressDevice;
	SPPClient _SPPClient = new SPPClient();
	
	VirtualTouchPad _padTurrel = null;
	Virtual5DPad _padTank = null;

	DisplayView _viewDisplay = null;
	int _iReadyStep = 0;
	
	boolean _bMultiTouch = false;
	
	int _iBody_Speed=50, _iBody_MinSpeed=0, _iBody_MidSpeed=50, _iBody_MaxSpeed=100;
	int _iTurrel_MinLow=10, _iTurrel_MidLow=80, _iTurrel_MaxLow=170;
	int _iTurrel_MinHigh=30, _iTurrel_MidHigh=85, _iTurrel_MaxHigh=105;
	int _iTurrel_MinDist=25, _iTurrel_MaxDist=145;

	boolean _bInScanMode = false;
	protected static final int SCANPROGRESS_START = 0;
	protected static final int SCANPROGRESS_MSG = 1;
	protected static final int SCANPROGRESS_FINISH = 2;
	protected ProgressDialog m_mScanProgressDialog = null;
	
	PowerManager.WakeLock wl = null;

	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.remote);

        FrameLayout frameDisplay = (FrameLayout) findViewById(R.id.frameDisplay);
        _viewDisplay = new DisplayView(this);
		frameDisplay.addView(_viewDisplay);
		_viewDisplay.setOnTouchListener(new OnTouchListener()
		{
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				
				if(!_viewDisplay._bReady) return true;
				
				if(_bMultiTouch)
				{
					for (int i = 0; i < event.getPointerCount(); i++) {
						if (event.getAction() == MotionEvent.ACTION_DOWN)
				        {   
							// push somewhere !
							_viewDisplay._padScanLine.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padRadar.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padHalf.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padQuick.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padReport.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padBody.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padBodySpeed.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
							_viewDisplay._padReinitTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_PRESSED);
				        }
						else if(event.getAction() == MotionEvent.ACTION_MOVE) 
				        {
							// move !
							_viewDisplay._padScanLine.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);							
							_viewDisplay._padRadar.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
							_viewDisplay._padHalf.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
							_viewDisplay._padQuick.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);							
							_viewDisplay._padReport.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
							_viewDisplay._padBody.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
							_viewDisplay._padBodySpeed.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
							_viewDisplay._padTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
							_viewDisplay._padReinitTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_MOVED);
				        }
						else if(event.getAction() == MotionEvent.ACTION_UP) 
				        {
							// release pressure !
							_viewDisplay._padScanLine.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padRadar.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padHalf.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padQuick.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padReport.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padBody.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padBodySpeed.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
							_viewDisplay._padReinitTurrel.setTouchEvent((int)event.getX(i), (int)event.getY(i), event.getPointerId(i), VirtualTouchPad.TOUCH_RELEASED);
				        }
					}
				}
				else
				{
					if (event.getAction() == MotionEvent.ACTION_DOWN)
			        {   
						// push somewhere !
						_viewDisplay._padScanLine.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padRadar.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padHalf.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padQuick.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padReport.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padBody.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padBodySpeed.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padTurrel.setTouchEvent((int)event.getX(), (int)event.getY(),0, VirtualTouchPad.TOUCH_PRESSED);
						_viewDisplay._padReinitTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_PRESSED);
			        }
					else if(event.getAction() == MotionEvent.ACTION_MOVE) 
			        {
						// move !
						_viewDisplay._padScanLine.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padRadar.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padHalf.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padQuick.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padReport.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padBody.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padBodySpeed.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
						_viewDisplay._padReinitTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_MOVED);
			        }
					else if(event.getAction() == MotionEvent.ACTION_UP) 
			        {
						// release pressure !
						_viewDisplay._padScanLine.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padRadar.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padHalf.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padQuick.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padReport.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padBody.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padBodySpeed.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
						_viewDisplay._padReinitTurrel.setTouchEvent((int)event.getX(), (int)event.getY(), 0, VirtualTouchPad.TOUCH_RELEASED);
			        }
				}
				
				if(_viewDisplay._padScanLine.haveNewEvent())
				{
					if(_viewDisplay._padScanLine.getButton() && !_bInScanMode)
					{
						mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_START, null));

						// do a scan line radar !
						if(!_viewDisplay._padHalf.isLocked() && !_viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanLine((short)_iTurrel_MinLow, (short)_iTurrel_MaxLow, (short)1, (short)_iTurrel_MidHigh));
						else if(!_viewDisplay._padHalf.isLocked() && _viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanLine((short)_iTurrel_MinLow, (short)_iTurrel_MaxLow, (short)5, (short)_iTurrel_MidHigh));
						else if(_viewDisplay._padHalf.isLocked() && !_viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanLine((short)(_iTurrel_MidLow-35), (short)(_iTurrel_MidLow+35), (short)1, (short)_iTurrel_MidHigh));
						else if(_viewDisplay._padHalf.isLocked() && _viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanLine((short)(_iTurrel_MidLow-35), (short)(_iTurrel_MidLow+35), (short)5, (short)_iTurrel_MidHigh));
					}
					
					_viewDisplay._padScanLine.resetNewEvent();
				}
				if(_viewDisplay._padRadar.haveNewEvent())
				{
					if(_viewDisplay._padRadar.getButton() && !_bInScanMode)
					{
						mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_START, null));
						
						// do a scan radar !
						if(!_viewDisplay._padHalf.isLocked() && !_viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanZone((short)_iTurrel_MinLow, (short)_iTurrel_MaxLow, (short)1, (short)_iTurrel_MinHigh, (short)_iTurrel_MaxHigh, (short)1));
						else if(!_viewDisplay._padHalf.isLocked() && _viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanZone((short)_iTurrel_MinLow, (short)_iTurrel_MaxLow, (short)5, (short)_iTurrel_MinHigh, (short)_iTurrel_MaxHigh, (short)5));
						else if(_viewDisplay._padHalf.isLocked() && !_viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanZone((short)(_iTurrel_MidLow-35), (short)(_iTurrel_MidLow+35), (short)1, (short)(_iTurrel_MidHigh-25), (short)(_iTurrel_MidHigh+25), (short)1));
						else if(_viewDisplay._padHalf.isLocked() && _viewDisplay._padQuick.isLocked())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelScanZone((short)(_iTurrel_MidLow-35), (short)(_iTurrel_MidLow+35), (short)5, (short)(_iTurrel_MidHigh-25), (short)(_iTurrel_MidHigh+25), (short)5));
					}
					
					_viewDisplay._padRadar.resetNewEvent();
				}
				if(_viewDisplay._padReport.haveNewEvent())
				{
					if(_viewDisplay._padReport.getButton())
					{
						_viewDisplay._bReportTransmission = _viewDisplay._padReport.isLocked();
						
						// enable/stop report transmission
						_SPPClient.sendMessage(BluetoothCommandProtocol.sendSetSendStatus(_viewDisplay._bReportTransmission));
					}
					
					_viewDisplay._padReport.resetNewEvent();
				}
				if(_viewDisplay._padBody.haveNewEvent())
				{
					if(_viewDisplay._padBody.getTouch()<0.5f || _viewDisplay._padBody.getButtonCenter())
					{
						// stop tout mouvement du vehicule !
						_SPPClient.sendMessage(BluetoothCommandProtocol.sendResetBody());
					}
					else
					{
						// ondule ton corps
						if(_viewDisplay._padBody.getButtonUp())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendBodyFront(_iBody_Speed)); // go front
						else if(_viewDisplay._padBody.getButtonDown())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendBodyBack(_iBody_Speed)); // go back
						else if(_viewDisplay._padBody.getButtonLeft())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendBodyLeft(_iBody_Speed)); // rotate left
						else if(_viewDisplay._padBody.getButtonRight())
							_SPPClient.sendMessage(BluetoothCommandProtocol.sendBodyRight(_iBody_Speed)); // rotate right
					}
					_viewDisplay._padBody.resetNewEvent();
					_viewDisplay.invalidate();
				}
				if(_viewDisplay._padBodySpeed.haveNewEvent())
				{
					if(_viewDisplay._padBodySpeed.getTouch()<0.5f)
					{
						// set vehicule speed !
						_iBody_Speed = _iBody_MinSpeed+(int)((1.0-_viewDisplay._padBodySpeed.getPosition())*_iBody_MaxSpeed);
					}
					_viewDisplay._padBodySpeed.resetNewEvent();
					_viewDisplay.invalidate();
				}
				if(_viewDisplay._padReinitTurrel.haveNewEvent())
				{
					if(_viewDisplay._padReinitTurrel.getButton())
					{
						// reinit turrel
						_SPPClient.sendMessage(BluetoothCommandProtocol.sendResetTurrel());
						_viewDisplay._padTurrel.reset();
						_viewDisplay.invalidate();
					}
					_viewDisplay._padReinitTurrel.resetNewEvent();
				}
				if(_viewDisplay._padTurrel.haveNewEvent())
				{
					if(_viewDisplay._padTurrel.getTouch()>0.5f)
					{
						short iLow = (short) _iTurrel_MidLow;
						short iHigh = (short) _iTurrel_MidHigh;
						
						float fL = _viewDisplay._padTurrel.getX();
						float fH = _viewDisplay._padTurrel.getY();
						
						if(fL>0)		iLow += (short) ((_iTurrel_MaxLow-_iTurrel_MidLow)*fL);
						else if(fL<0)	iLow -= (short) ((_iTurrel_MinLow-_iTurrel_MidLow)*fL);
						
						if(fH>0)		iHigh += (short) ((_iTurrel_MaxHigh-_iTurrel_MidHigh)*fH);
						else if(fH<0)	iHigh -= (short) ((_iTurrel_MinHigh-_iTurrel_MidHigh)*fH);

						_SPPClient.sendMessage(BluetoothCommandProtocol.sendTurrelSetPosition(iLow, iHigh));
					}
					_viewDisplay._padTurrel.resetNewEvent();
					_viewDisplay.invalidate();
				}
		        // consume motion event
		        return true;
			}
		});
		
		Bundle bundle = this.getIntent().getExtras();
		_sAddressDevice = bundle.getString(BT_ADDRESS_DEVICE);
    }

	final Handler mBTHandler = new Handler() {
	    @Override
		public void handleMessage(Message msg) {
	        switch (msg.what)
	        {
				case SPPClient.MESSAGE_CONNECTED:
				{
					Toast.makeText(getApplicationContext(), "Connected",
							Toast.LENGTH_SHORT);
					_viewDisplay._bOnline = true;
					_viewDisplay.setTargetMessage("Online...");
					_SPPClient.sendMessage(BluetoothCommandProtocol.sendAskVersion()); // ask version
					_SPPClient.sendMessage(BluetoothCommandProtocol.sendAskSetting()); // check report sending.
					_SPPClient.sendMessage(BluetoothCommandProtocol.sendAskSettingBody()); // get body setting
					_SPPClient.sendMessage(BluetoothCommandProtocol.sendAskSettingTurrel()); // get turrel setting
					break;
				}
		        case SPPClient.MESSAGE_DECONNECTED:
		        {
		        	Toast.makeText(getApplicationContext(), "Disconnected", Toast.LENGTH_SHORT);
		        	_viewDisplay._bOnline = false;
		        	_viewDisplay._bReady = false;
		            break;
		        }
		        case SPPClient.MESSAGE_GETERROR:
		        {
		        	Toast.makeText(getApplicationContext(), "Get Error : " + (String)msg.obj, Toast.LENGTH_LONG);
		        	break;
		        }
		        case SPPClient.MESSAGE_GETTEXT:
		        {
		        	String sMessage = (String)msg.obj;
		        	Toast.makeText(getApplicationContext(), "Get text : " + sMessage, Toast.LENGTH_LONG);
	        		break;
		        }
		        case SPPClient.MESSAGE_GETCMD:
		        {
		        	String sMessage = (String)msg.obj;
		        	Log.d(TAG, "Get cmd : "+sMessage);
	        		String [] sCmds = sMessage.split("\\:");
	        		if(sCmds.length==2)
	        		{
	        			String sCmd = sCmds[0].trim().toLowerCase();
	        			String sOps = sCmds[1].trim().toLowerCase();
	        			if(sCmd.equals("version"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			if(sVals.length != 2) continue;
			        				
			        			String name = sVals[0].trim();
			        			//String val = sVals[1].trim();
			        			
			        			// TODO get name and protocol version
			        			if(name.equals("name"))
			        			{
			        				// name of robot ?
			        			}
			        			else if(name.equals("ver"))
			        			{
			        				// version of robot ?
			        			}
			        		}
			        		_iReadyStep |= 0x01;
			        	}
			        	else if(sCmd.equals("setting"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();
			        			
			        			if(name.equals("send_status"))
			        			{
			        				_viewDisplay._bReportTransmission =  val.equals("1")?true:false;
			        				_viewDisplay._padReport.setLock(_viewDisplay._bReportTransmission);
			        			}
			        		}
			        		_viewDisplay.invalidate();			        		
			        		_iReadyStep |= 0x02;
			        		
			        		if(!_viewDisplay._bReportTransmission)
			        			_SPPClient.sendMessage(BluetoothCommandProtocol.sendSetSendStatus(true));
			        	}
			        	else if(sCmd.equals("setting_body"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();
			        			
								if(name.equals("min_speed"))
			        				_iBody_MinSpeed =  Integer.parseInt(val);
			        			else if(name.equals("mid_speed"))
			        				_iBody_MidSpeed =  Integer.parseInt(val);
			        			else if(name.equals("max_speed"))
			        				_iBody_MaxSpeed =  Integer.parseInt(val);
			        		}
			        		
			        		_viewDisplay._padBodySpeed.setPosition((float)(_iBody_MaxSpeed-_iBody_MinSpeed)/(float)(_iBody_MidSpeed-_iBody_MinSpeed));
			        		_viewDisplay.invalidate();
			        		_iReadyStep |= 0x04;
			        	}
			        	else if(sCmd.equals("setting_turrel"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();

			        			if(name.equals("min_low"))
			        				_iTurrel_MinLow =  Integer.parseInt(val);
			        			else if(name.equals("mid_low"))
			        				_iTurrel_MidLow =  Integer.parseInt(val);
			        			else if(name.equals("max_low"))
			        				_iTurrel_MaxLow =  Integer.parseInt(val);
			        			else if(name.equals("min_high"))
			        				_iTurrel_MinHigh =  Integer.parseInt(val);
			        			else if(name.equals("mid_high"))
			        				_iTurrel_MidHigh =  Integer.parseInt(val);
			        			else if(name.equals("max_high"))
			        				_iTurrel_MaxHigh =  Integer.parseInt(val);
			        		}
			        		_iReadyStep |= 0x08;
			        		
			        		// prepare size of scan
			        	}
			        	else if(sCmd.equals("sensors"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();
			        			
			        			if(name.equals("fl")) _viewDisplay._bFrontLeftSwitch = val.equals("1")?true:false;
			        			else if(name.equals("fr")) _viewDisplay._bFrontRightSwitch = val.equals("1")?true:false;
			        			else if(name.equals("bl")) _viewDisplay._bBackLeftSwitch = val.equals("1")?true:false;
			        			else if(name.equals("br")) _viewDisplay._bBackRightSwitch = val.equals("1")?true:false;
			        		}
			        		_viewDisplay.invalidate();
			        	}
			        	else if(sCmd.equals("turrel"))
			        	{
			        		String [] sDatas = sOps.split("\\,");
			        		for(String d : sDatas)
			        		{
			        			String [] sVals = d.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();

			        			if(name.equals("low")) _viewDisplay._iTurrelLow = Integer.parseInt(val);
			        			else if(name.equals("high")) _viewDisplay._iTurrelHigh = Integer.parseInt(val);
			        			else if(name.equals("dist")) _viewDisplay._iTurrelDistance = Integer.parseInt(val);
			        		}
			        		_viewDisplay.invalidate();			        		
			        	}
			        	else if(sCmd.equals("scan_line"))
			        	{
		        			int low_step = 1;
		        			int low_start = _iTurrel_MinLow, low_end = _iTurrel_MaxLow;

		        			mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_START, null));
		        			mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_MSG, "Analysing datas..."));

		        			String [] sParams = sOps.split("\\,");
			        		for(String p : sParams)
			        		{
			        			String [] sVals = p.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();
			        			
			        			if(name.equals("low_step")) low_step = Integer.parseInt(val);
			        			else if(name.equals("low_start")) low_start = Integer.parseInt(val);
			        			else if(name.equals("low_end")) low_end = Integer.parseInt(val);
			        			else if(name.equals("data"))
			        			{
			        				Bitmap bitmap = Bitmap.createBitmap(low_end-low_start, 5, Bitmap.Config.ARGB_8888);
			        				
					        		String [] sDatas = val.split("\\;");
					        		for(String d : sDatas)
					        		{
					        			String [] sD = d.split("\\ ");
					        			if(sD.length != 3) continue;
					        			
					        			String sL = sD[0].trim();
					        			String sH = sD[1].trim();
					        			String sV = sD[2].trim();
		
					        			int l = Integer.parseInt(sL);
					        			int h = Integer.parseInt(sH);
					        			int v = Integer.parseInt(sV);
					        			
					        			if(l>=_iTurrel_MinLow && l<=_iTurrel_MaxLow
					        				&& h>=_iTurrel_MinHigh && h<=_iTurrel_MaxHigh)
					        			{
					        				int c = Color.RED;
					        				if(v<_iTurrel_MinDist) c = Color.DKGRAY;
					        				else if(v>_iTurrel_MaxDist) c = Color.BLACK;
					        				else
					        				{
					        					float fC = 1.0f-((float)(v-_iTurrel_MinDist)/(float)(_iTurrel_MaxDist-_iTurrel_MinDist));
					        					int a = (int)(fC*255);
					        					c = Color.argb(255, a/2, a/2, a);
					        				}
					        				l = low_end-(l-low_start);
					        				for(int x=0; x<low_step && (l+x-low_start)<bitmap.getWidth(); x++)
					        				{
					        					bitmap.setPixel(l+x-low_start, 1, c);
					        					bitmap.setPixel(l+x-low_start, 2, c);
					        					bitmap.setPixel(l+x-low_start, 3, c);
					        				}
					        			}
					        		}
					        		_viewDisplay.prepareImageRadar(bitmap);
				        		}
			        		}
		    				mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_FINISH, null));		        		
			        	}
			        	else if(sCmd.equals("scan_zone"))
			        	{
		        			int low_step = 1, high_step = 1;
		        			int low_start = _iTurrel_MinLow, low_end = _iTurrel_MaxLow;
		        			int high_start = _iTurrel_MinHigh, high_end = _iTurrel_MaxHigh;

		        			mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_START, null));
		        			mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_MSG, "Analysing datas..."));
		        			
		        			String [] sParams = sOps.split("\\,");
			        		for(String p : sParams)
			        		{
			        			String [] sVals = p.split("\\=");
			        			if(sVals.length != 2) continue;
			        			
			        			String name = sVals[0].trim();
			        			String val = sVals[1].trim();
			        			
			        			if(name.equals("low_step")) low_step = Integer.parseInt(val);
			        			else if(name.equals("high_step")) high_step = Integer.parseInt(val);
			        			else if(name.equals("low_start")) low_start = Integer.parseInt(val);
			        			else if(name.equals("low_end")) low_end = Integer.parseInt(val);
			        			else if(name.equals("high_start")) high_start = Integer.parseInt(val);
			        			else if(name.equals("high_end")) high_end = Integer.parseInt(val);
			        			else if(name.equals("data"))
			        			{
			        				Bitmap bitmap = Bitmap.createBitmap(low_end-low_start, high_end-high_start, Bitmap.Config.ARGB_8888);
			        				
					        		String [] sDatas = val.split("\\;");
					        		for(String d : sDatas)
					        		{
					        			String [] sD = d.split("\\ ");
					        			if(sD.length != 3) continue;
					        			
					        			String sL = sD[0].trim();
					        			String sH = sD[1].trim();
					        			String sV = sD[2].trim();
		
					        			int l = Integer.parseInt(sL);
					        			int h = Integer.parseInt(sH);
					        			int v = Integer.parseInt(sV);
					        			
					        			if(l>=_iTurrel_MinLow && l<=_iTurrel_MaxLow
					        				&& h>=_iTurrel_MinHigh && h<=_iTurrel_MaxHigh)
					        			{
					        				int c = Color.RED;
					        				if(v<_iTurrel_MinDist) c = Color.DKGRAY;
					        				else if(v>_iTurrel_MaxDist) c = Color.BLACK;
					        				else
					        				{
					        					float fC = 1.0f-((float)(v-_iTurrel_MinDist)/(float)(_iTurrel_MaxDist-_iTurrel_MinDist));
					        					int a = (int)(fC*255);
					        					c = Color.argb(255, a/2, a/2, a);
					        				}
					        				
					        				l = low_end-(l-low_start);
					        				h = high_end-(h-high_start);
					        				for(int x=0; x<low_step && (l+x-low_start)<bitmap.getWidth(); x++)
					        				{
						        				for(int y=0; y<high_step && (h+y-high_start)<bitmap.getHeight(); y++)
						        				{
						        					bitmap.setPixel(l+x-low_start, h+y-high_start, c);
						        				}
					        				}
					        			}
					        		}
					        		_viewDisplay.prepareImageRadar(bitmap);					        		
			        			}
			        		}
			        		mScanProgressHandler.sendMessage(mScanProgressHandler.obtainMessage(SCANPROGRESS_FINISH, null));
			        	}
			        	else
			        		Toast.makeText(getApplicationContext(), "Unknown cmd : " + (String)msg.obj, Toast.LENGTH_LONG);
	        		}
		        	else
		        	{
			        	Toast.makeText(getApplicationContext(), "Receive : " + (String)msg.obj, Toast.LENGTH_LONG);
		        	}
	        		
	        		if(_viewDisplay._bReady == false && _iReadyStep == (0x01|0x02|0x04|0x08))
	        		{
	        			_viewDisplay._bReady = true;
	        			_viewDisplay.setTargetMessage("");
	        		}
		            break;
		        }
	        }
	    }
	};
	
	@Override
	public void onResume()
	{
		super.onResume();
		
		if(wl == null)
		{
			PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
		    wl = pm.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK, "Remote Robot");
		}
		if(wl != null ) wl.acquire();
		
		_viewDisplay.setTargetMessage("Try connection...");
		if(!_SPPClient.isConnected())
		{
			if(!_SPPClient.connect(_sAddressDevice, mBTHandler))
			{
				Toast.makeText(this, "Cannot connect :(", Toast.LENGTH_LONG);    	
				_viewDisplay.setTargetMessage("Connection impossible !!!");
			}
		}
	}
	
	@Override
	public void onPause()
	{
		super.onPause();
		
		if(wl != null ) wl.release();
	}

	final Handler mScanProgressHandler = new Handler() {
	    @Override
		public void handleMessage(Message msg) {
	        switch (msg.what) {
	        case SCANPROGRESS_START:
	        	if(_bInScanMode)
	        		break;
	        	
	        	if(m_mScanProgressDialog != null)
	        		m_mScanProgressDialog.dismiss();
            	m_mScanProgressDialog = ProgressDialog.show(Remote_Activity.this, "Scanning", "wait...", true);

	            _bInScanMode = true;
	            break;
	        case SCANPROGRESS_MSG:
	            if (m_mScanProgressDialog!=null && m_mScanProgressDialog.isShowing()) {
	            	m_mScanProgressDialog.setMessage(((String) msg.obj));
	            }
	            break;
	        case SCANPROGRESS_FINISH:
	            if (m_mScanProgressDialog!=null && m_mScanProgressDialog.isShowing()) {
	            	m_mScanProgressDialog.dismiss();
	            	m_mScanProgressDialog = null;
	            }
	            _bInScanMode = false;
	            break;
	        }
	    }
	};
}
